﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace EncryptFileRijndaelSample
{
    class Program
    {
        static void Main(string[] args)
        {
            string inputFile = "GeheimeDatei.txt";
            string outputFileEncrypted = @"c:\Temp\GeheimeDateiVerschluesselt.txt";
            string outputFileDecrypted = @"c:\Temp\GeheimeDateiEntschluesselt.txt";
            string keyFile = @"c:\Temp\Key.dat";

            EncryptFile(inputFile, outputFileEncrypted, keyFile);

            DecryptFile(outputFileEncrypted, outputFileDecrypted, keyFile);
        }

        ///<summary>
        /// Datei mit dem Rijndael-Algorithmus verschlüsseln
        ///</summary>
        ///<param name="inputFile">Zu verschlüsselnde Datei</param>
        ///<param name="outputFile">Neue verschlüssselte Datei</param>
        ///<param name="keyFile">Schlüssel für die Verschlüsselung</param>
        private static void EncryptFile(string inputFile, string outputFile, string keyFile)
        {
            try
            {
                string cryptFile = outputFile;
                RijndaelManaged RMCrypto = new RijndaelManaged();

                using (FileStream fsCrypt = new FileStream(cryptFile, FileMode.Create))
                {
                    using (CryptoStream cs = new CryptoStream(fsCrypt, RMCrypto.CreateEncryptor(), CryptoStreamMode.Write))
                    {
                        byte[] fileContent = File.ReadAllBytes(inputFile);
                        cs.Write(fileContent, 0, fileContent.Length);                        

                        // Verwendeten Schlüssel exportieren
                        File.WriteAllBytes(keyFile, RMCrypto.Key);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Verschlüsselung fehlgeschlagen!");
                Console.WriteLine($"Fehler: {ex.Message}");
            }
        }

        ///<summary>
        /// Verschlüsselte Datei mit dem Rijndael-Algorithmus entschlüsseln
        ///</summary>
        ///<param name="inputFile">Die verschlüsselte Datei</param>
        ///<param name="outputFile">Neue entschlüsselte Datei</param>
        ///<param name="keyFile">Schlüssel für die Entschlüsselung</param>
        private static void DecryptFile(string inputFile, string outputFile, string keyFile)
        {
            try
            {
                RijndaelManaged RMCrypto = new RijndaelManaged();
                RMCrypto.Key = File.ReadAllBytes(keyFile);

                using (FileStream fsCrypt = new FileStream(inputFile, FileMode.Open))
                {
                    using (CryptoStream cs = new CryptoStream(fsCrypt, RMCrypto.CreateDecryptor(), CryptoStreamMode.Read))
                    {                       
                        using (FileStream fsOut = new FileStream(outputFile, FileMode.Create))
                        {  
                            int data;
                            while ((data = cs.ReadByte()) != -1)
                            {
                                fsOut.WriteByte((byte)data);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Entschlüsselung fehlgeschlagen!");
                Console.WriteLine($"Fehler: {ex.Message}");
            }
        }        
    }
}
