﻿using System;
using System.IO;

namespace DriveInfoSample01
{
    class Program
    {
        static void Main(string[] args)
        {
            DriveInfo[] allDrives = DriveInfo.GetDrives();

            foreach (DriveInfo d in allDrives)
            {
                Console.WriteLine($"Laufwerk {d.Name}");
                if (d.IsReady == true)
                {
                    Console.WriteLine($"  Label: {d.VolumeLabel}");
                    Console.WriteLine($"  Typ: {d.DriveType}");
                    Console.WriteLine($"  Dateisystem: {d.DriveFormat}");
                    Console.WriteLine(
                        $"  Freier Speicher: {ToReadableSize(d.AvailableFreeSpace)}");

                    Console.WriteLine(
                        $"  Belegter Speicher: {ToReadableSize(d.TotalSize - d.AvailableFreeSpace)}");

                    Console.WriteLine(
                        $"  Gesamtkapazität: {ToReadableSize(d.TotalSize)}");
                }
            }
        }

        /// <summary>
        /// Methode zur Formatierung des Speicherplatzes
        /// </summary>
        /// <param name="number">Größe in bytes</param>
        /// <param name="precision">Anzahl Nachkommastellen</param>
        /// <returns></returns>
        public static string ToReadableSize(double number, int precision = 2)
        {
            string[] suffixes = new[] { " B", " KB", " MB", " GB", " TB", " PB" };

            // Umrechnungsfaktor
            const double unit = 1024;
            // Zähler für das Suffix
            int i = 0;
            // Prüfen in welcher Einheit wir uns befinden
            while (number > unit)
            {
                number /= unit;
                i++;
            }

            // Anzahl Nachkommastellen und Suffix festlegen
            return Math.Round(number, precision) + suffixes[i];
        }
    }
}
