﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TaskCancellationTokenSample01
{
    class Program
    {
        static void Main(string[] args)
        {
            // CancellationTokenSource Instanz erstellen
            CancellationTokenSource cts = new CancellationTokenSource();
            CancellationToken ct = cts.Token;

            // Hier kann eine Methode registriert werden, die
            // ausgeführt wird sobald ein Abbruch erfolgt! 
            ct.Register(() => Console.WriteLine("Erfolgreich abgebrochen!"));

            Task t = new Task(() =>
            {
                for (int i = 0; i < 100000; i++)
                {
                    if (ct.IsCancellationRequested)
                    {
                        Console.WriteLine($"In Iteration" +
                            $" {i + 1}, wurde abgebrochen ...");

                        // Hier dann Aufräumarbeiten für
                        // den Abbruch und Exception auslösen
                        ct.ThrowIfCancellationRequested();
                        break;
                    }

                    // Warten um Arbeit zu simulieren
                    Thread.Sleep(25);
                }                          
            }, ct);  // Token mitgeben
            
            try
            {
                // Starten
                t.Start();
                // Task nach 1 Sek. abbrechen
                cts.CancelAfter(1000);
                // Auf den Task warten
                t.Wait();
            }
            catch (AggregateException ex)
            {
                Console.WriteLine(ex.InnerException.Message);
            }
            finally
            {
                cts.Dispose();
            }

            Console.WriteLine($"Cancelled: {t.IsCanceled}, " +
                $"Finished: {t.IsCompleted}, " +
                $"Error: {t.IsFaulted}");

            Console.ReadKey();
        }       
    }
}
