﻿using Npgsql;
using NpgsqlTypes;
using System;
using System.Data;

namespace ADONetDataSetSample02
{
    class Program
    {
        static string connString = "Host=localhost;" +
            "Username=postgres;Password=postgres;" +
            "Database=Northwind";
        static void Main(string[] args)
        {
            // DataSet erstellen
            DataSet ds = new DataSet();

            // Connection erstellen
            NpgsqlConnection conn = new NpgsqlConnection(connString);          

            // DataAdapter erzeugen
            NpgsqlDataAdapter da = new NpgsqlDataAdapter();        
            da.SelectCommand = CreateCategorySelectCmd(conn);
            da.InsertCommand = CreateCategoryInsertCmd(conn);
            da.UpdateCommand = CreateCategoryUpdateCmd(conn);
            da.DeleteCommand = CreateCategoryDeleteCmd(conn);

            da.Fill(ds, "Categories");

            // Datensatz editieren
            DataRow row = ds.Tables["Categories"].Rows[0];
            row["category_name"] = "Neuer Name";
            row["description"] = "Neue Beschreibung";

            // Datensatz einfügen
            DataRow newRow = ds.Tables["Categories"].NewRow();
            newRow["category_id"] = 20;
            newRow["category_name"] = "Meine Kategorie";
            newRow["description"] = "Neu angelegte Kategorie";
            ds.Tables["Categories"].Rows.Add(newRow);

            // Datensatz löschen
            //DataRow deltedRow = ds.Tables["Categories"].Rows[3];
            //deltedRow.Delete();

            da.Update(ds, "Categories");

        }

        static NpgsqlCommand CreateCategorySelectCmd(NpgsqlConnection conn)
        {
            NpgsqlCommand cmd = new NpgsqlCommand("SELECT category_id, " +
                "category_name, description FROM categories", conn);

            return cmd;
        }

        static NpgsqlCommand CreateCategoryUpdateCmd(NpgsqlConnection conn)
        {
            string cmdText = 
                "UPDATE categories " +
                "SET category_name = @p1," +
                "    description   = @p2 " +
                "WHERE category_id = @p3";

            NpgsqlCommand cmd = new NpgsqlCommand(cmdText, conn);

            cmd.Parameters.Add("@p1", NpgsqlDbType.Varchar, 15, "category_name");
            cmd.Parameters.Add("@p2", NpgsqlDbType.Text, 0, "description");
            cmd.Parameters.Add("@p3", NpgsqlDbType.Smallint, 0, "category_id");

            return cmd;
        }

        static NpgsqlCommand CreateCategoryDeleteCmd(NpgsqlConnection conn)
        {
            string cmdText =
                "DELETE FROM categories " +                
                "WHERE category_id = @p1";

            NpgsqlCommand cmd = new NpgsqlCommand(cmdText, conn);

            cmd.Parameters.Add("@p1", NpgsqlDbType.Smallint, 0, "category_id");

            return cmd;
        }

        static NpgsqlCommand CreateCategoryInsertCmd(NpgsqlConnection conn)
        {
            string cmdText =
                "INSERT INTO categories " +
                "(category_id, category_name, description) " +
                "VALUES (@p1, @p2, @p3)";

            NpgsqlCommand cmd = new NpgsqlCommand(cmdText, conn);

            cmd.Parameters.Add("@p1", NpgsqlDbType.Smallint, 0, "category_id");
            cmd.Parameters.Add("@p2", NpgsqlDbType.Varchar, 15, "category_name");
            cmd.Parameters.Add("@p3", NpgsqlDbType.Text, 0, "description");

            return cmd;
        }
    }
}