﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace WpfMVVMSample01.Base
{
    public abstract class BindableBase : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
		/// Prüft, ob die Eigenschaft schon den gewünschten Wert hat. 
        /// Setzt die Eigenschaft und benachrichtigt wenn nötig.
		/// </summary>
		/// <typeparam name="T">Typ der Eigenschaft</typeparam>
		/// <param name="storage">Referenz auf den aktuellen Wert</param>
		/// <param name="value">Gewünschter Wert</param>
		/// <param name="propertyName">Name der Eigenschaft für die 
		/// Benachrichtigung. Wert ist optional und wird autom. gesetzt.
		/// </param>
		/// <returns>True wenn sich der Wert geändert hat, false wenn
        /// der Wert schon dem gewünschten Wert entspricht.</returns>
        protected virtual bool SetProperty<T>(ref T storage, 
            T value, [CallerMemberName] string propertyName = null)
        {
            if (EqualityComparer<T>.Default.Equals(storage, value))
                return false;

            storage = value;
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));

            return true;
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
        {
            PropertyChanged?.Invoke(this, args);
        }
    }
}