﻿using ModularApplicationSample.Core;
using ModularApplicationSample.Infrastructure.Interfaces;
using ModularApplicationSample.Infrastructure.Model;
using Prism.Commands;
using Prism.Mvvm;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ModularApplicationSample.PLZ.ViewModels
{
    public class PLZManagementViewModel : ViewModelBase
    {
        public PLZManagementViewModel(IPLZManagementService plzManagementService)
        {
            this.Title = "PLZ-Management";
            this.Icon = "Map";

            this.PLZManagementService = plzManagementService;

            // Commands initialisieren
            this.InitializeCommands();

            // Daten laden
            this.LoadData();
        }

        private void LoadData()
        {
            // Service-Methode zum Laden der PLZ
            this.Postleitzahlen = this.PLZManagementService.GetAllPLZ();
        }

        private void InitializeCommands()
        {
            this.OpenInGoogleMapsCommand = new DelegateCommand<string>
                (this.OpenInGoogleMapsCommandExecute, 
                this.OpenInGoogleMapsCommandCanExecute)
                .ObservesProperty<string>(() => this.SelectedPLZ.PLZ);
        }

        public DelegateCommand<string> OpenInGoogleMapsCommand { get; private set; }

        private void OpenInGoogleMapsCommandExecute(string plz)
        {
            System.Diagnostics
                .Process
                .Start($"https://www.google.de/maps/place/{plz}");
        }

        private bool OpenInGoogleMapsCommandCanExecute(string plz)
        {
            return !String.IsNullOrEmpty(plz);
        }

        public IPLZManagementService PLZManagementService { get; private set; }

        private IList<Postleitzahl> postleitzahlen;
        public IList<Postleitzahl> Postleitzahlen
        {
            get { return postleitzahlen; }
            set { SetProperty(ref postleitzahlen, value); }
        }

        private Postleitzahl selectedPLZ;
        public Postleitzahl SelectedPLZ
        {
            get { return selectedPLZ; }
            set { SetProperty(ref selectedPLZ, value); }
        }
    }
}