﻿using ModularApplicationSample.Views;
using Prism.Ioc;
using Prism.Modularity;
using Prism.Regions;
using System.Windows;
using ModularApplicationSample.Core.Constants;
using ModularApplicationSample.RegionAdapter;
using System.Windows.Controls;
using MahApps.Metro.Controls;

namespace ModularApplicationSample
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App
    {
        protected override Window CreateShell()
        {
            return Container.Resolve<MainWindow>();
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            containerRegistry.RegisterForNavigation<Home>("Home");
        }

        protected override void ConfigureRegionAdapterMappings(RegionAdapterMappings regionAdapterMappings)
        {
            base.ConfigureRegionAdapterMappings(regionAdapterMappings);

            // Registrierung des eigenen Region-Adapters
            var regionBehaviorFactory = Container.Resolve<IRegionBehaviorFactory>();
            regionAdapterMappings.RegisterMapping(typeof(MetroTabControl), new MetroTabControlRegionAdapter(regionBehaviorFactory));
        }

        protected override void InitializeShell(Window shell)
        {
            base.InitializeShell(shell);

            // Erstellen des Übersichts-Views
            Container.Resolve<IRegionManager>().RequestNavigate(RegionNames.MainRegion, "Home");
        }

        protected override void ConfigureModuleCatalog(IModuleCatalog moduleCatalog)
        {
            base.ConfigureModuleCatalog(moduleCatalog);

            // Modul mit den Postleitzahlen registrieren
            // Danach wir autamtisch die Klasse PLZModule durchlaufen
            // und die dort registrierten Views werden dann in der Übersicht
            // angezeigt
            moduleCatalog.AddModule<ModularApplicationSample.PLZ.PLZModule>();
        }

    }
}
