﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EventSample01
{
    class Program
    {
        /// <summary>
        /// Klasse für die Event-Informationen
        /// </summary>
        public class TemperatureEventArgs : EventArgs
        {
            public TemperatureEventArgs(DateTime time, decimal temperature, string message)
            {
                Time = time;
                Temperature = temperature;
                Message = message;
            }

            public DateTime Time { get; set; } 
            public decimal Temperature { get; set; }
            public string Message { get; set; }
        }

        public class Heizung
        {
            // Ereignis bereitstellen
            public event EventHandler<TemperatureEventArgs> TemperatureChangedEvent;

            public void DoSomething()
            {
                // Daten für EventArgs
                DateTime time = DateTime.Now;
                decimal temperature = 35;

                // Hier würde normalerweise etwas sinnvolles
                // passieren bevor das Ereignis ausgelöst wird 
                OnRaiseCustomEvent(new TemperatureEventArgs(time, temperature, 
                    $"{time} - Heizung 1 Temperaturanstieg auf {temperature} °C!"));
            }

            // Eigene Methode, die das Ereinis auslöst
            protected virtual void OnRaiseCustomEvent(TemperatureEventArgs e)
            {
                EventHandler<TemperatureEventArgs> handler = TemperatureChangedEvent;

                // Ereignis könnte null sein, wenn es keine Abonnenten gibt
                handler?.Invoke(this, e);
            }
        }

        static void Main(string[] args)
        {
            var heizung = new Heizung();

            // Event registrieren
            heizung.TemperatureChangedEvent += Heizung_TemperatureChangedEvent;

            heizung.DoSomething();

            Console.ReadKey();
        }

        /// <summary>
        /// Event-Handler
        /// </summary>
        private static void Heizung_TemperatureChangedEvent(object sender, TemperatureEventArgs e)
        {
            Console.WriteLine(e.Message);
        }
    }
}
