﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LinqToObjects
{
    public class OrderService
    {
        private DateTime date = new DateTime(2018, 10, 20);

        public OrderService()
        {
            // Liste mit Produkten initialisieren
            InitProducts();
            // Liste mit Auftragspositionen initialisieren
            InitOrderItems();
            // Liste mit Aufträgen initialisieren
            InitOrderList();
            // Liste mit Kunden initialisieren
            InitCustomerList();
        }

        private void InitCustomerList()
        {
            Customers = new List<Customer>();

            Customers.Add(new Customer()
            {
                CustomerId = 1,
                FirstName = "Steffen",
                LastName = "Steinbrecher",
                Street = "Musterstrasse 11",
                City = "Musterstadt",
                Postcode = "12345",
                Country = "Germany",
                Orders = new List<Order>() { Orders[0], Orders[1] }
            });
            Customers.Add(new Customer()
            {
                CustomerId = 2,
                FirstName = "Max",
                LastName = "Mustermann",
                Street = "Hirschstrasse 11",
                City = "Hamburg",
                Postcode = "89642",
                Country = "Germany",
                Orders = new List<Order>() { Orders[2], Orders[3], Orders[4] }
            });
            Customers.Add(new Customer()
            {
                CustomerId = 3,
                FirstName = "Harry",
                LastName = "Hirsch",
                Street = "Dorfstrasse 8",
                City = "München",
                Postcode = "98547",
                Country = "Germany",
                Orders = new List<Order>() { Orders[5] }
            });
            Customers.Add(new Customer()
            {
                CustomerId = 4,
                FirstName = "Klaus",
                LastName = "Müller",
                Street = "Hauptstrasse 118",
                City = "Musterstadt",
                Postcode = "12345",
                Country = "Germany",
                Orders = new List<Order>() { Orders[6], Orders[7] }
            });
            Customers.Add(new Customer()
            {
                CustomerId = 5,
                FirstName = "Martin",
                LastName = "Müller",
                Street = "Am Eck 12",
                City = "Musterstadt",
                Postcode = "12345",
                Country = "Germany",
                Orders = new List<Order>() { Orders[8], Orders[9], Orders[10] }
            });
        }

        private void InitOrderList()
        {
            Orders = new List<Order>()
            {
               new Order() { OrderId = 1, OrderDate = date.AddDays(-10),
                             Shipped = true, ShippingDate = date.AddDays(-5),
                             Items = GetOrderItems(2) },
               new Order() { OrderId = 2, OrderDate = date.AddDays(-8), Shipped = true, ShippingDate = date.AddDays(-6), Items = GetOrderItems(4) },
               new Order() { OrderId = 3, OrderDate = date.AddDays(-5), Shipped = true, ShippingDate = date.AddDays(-4), Items = GetOrderItems(1) },
               new Order() { OrderId = 4, OrderDate = date.AddDays(-4), Shipped = false, ShippingDate = null, Items = GetOrderItems(5) },
               new Order() { OrderId = 5, OrderDate = date.AddDays(-3), Shipped = true, ShippingDate = date.AddDays(-3), Items = GetOrderItems(1) },
               new Order() { OrderId = 6, OrderDate = date.AddDays(-2), Shipped = true, ShippingDate = date.AddDays(-2), Items = GetOrderItems(3) },
               new Order() { OrderId = 7, OrderDate = date.AddDays(-2), Shipped = true, ShippingDate = date.AddDays(-2), Items = GetOrderItems(2) },
               new Order() { OrderId = 8, OrderDate = date.AddDays(-2), Shipped = true, ShippingDate = date.AddDays(-1), Items = GetOrderItems(1) },
               new Order() { OrderId = 9, OrderDate = date.AddDays(-1), Shipped = false, ShippingDate = null, Items = GetOrderItems(1) },
               new Order() { OrderId = 10, OrderDate = date.AddDays(-1), Shipped = false, ShippingDate = null, Items = GetOrderItems(1) },
               new Order() { OrderId = 11, OrderDate = date.AddDays(-1), Shipped = false, ShippingDate = null, Items = GetOrderItems(2) }
            };
        }

        private void InitProducts()
        {
            Products = new List<Product>()
                        {
                            new Product() { ProductId = 1, Name = "Huawai P10", Price = 315.49 },
                            new Product() { ProductId = 2, Name = "Huawai P9", Price = 298.98 },
                            new Product() { ProductId = 3, Name = "Apple Iphone 6", Price = 360.49 },
                            new Product() { ProductId = 4, Name = "Apple Iphone 6 Plus", Price = 399.99 },
                            new Product() { ProductId = 5, Name = "Apple Iphone 7", Price = 450.90 },
                            new Product() { ProductId = 6, Name = "Apple Iphone 7 Plus", Price = 499.99 },
                            new Product() { ProductId = 7, Name = "Samsung Galaxy S7", Price = 399.00 },
                            new Product() { ProductId = 8, Name = "Samsung Galayy S8", Price = 449.00 }
                        };
        }

        private void InitOrderItems()
        {
            OrderItems = new List<OrderItem>()
            {
                new OrderItem() { ProductId = 1, Quantity = 1 },
                new OrderItem() { ProductId = 2, Quantity = 1 },
                new OrderItem() { ProductId = 3, Quantity = 1 },
                new OrderItem() { ProductId = 4, Quantity = 1 },
                new OrderItem() { ProductId = 5, Quantity = 1 },
                new OrderItem() { ProductId = 6, Quantity = 1 },
                new OrderItem() { ProductId = 7, Quantity = 1 },
                new OrderItem() { ProductId = 8, Quantity = 1 }
            };
        }

        /// <summary>
        /// List with customers
        /// </summary>
        public IList<Customer> Customers { get; private set; }

        /// <summary>
        /// List with orders
        /// </summary>
        public IList<Order> Orders { get;  private set; }

        /// <summary>
        /// Liste mit Produkten
        /// </summary>
        public List<Product> Products { get;  private set; }

        private IList<OrderItem> OrderItems { get; set; }

        private IList<OrderItem> GetOrderItems(int numberOfItems)
        {
            var orderItems = new List<OrderItem>();

            if (numberOfItems <= OrderItems.Count)
            {
                for (int i = 0; i < numberOfItems; i++)
                {
                    orderItems.Add(OrderItems[i]);
                }
            }
            else
            {
                return OrderItems;
            }

            return orderItems;
        }
    }
}