﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CopyDirectory
{
    public static class FileSystemUtils
    {
        public static void DirectoryCopy(string sourceDirName, string destDirName, bool copySubDirs)
        {
            // DirectoryInfo-Objekt für das Quellverzeichnis
            DirectoryInfo dir = new DirectoryInfo(sourceDirName);

            // Existiert das Quellverzeichnis?
            if (!dir.Exists)
            {
                throw new DirectoryNotFoundException(
                  "Quellverzeichnis exisitert nicht oder kann nicht gefunden werden: "
                  + sourceDirName);
            }

            DirectoryInfo[] dirs = dir.GetDirectories();

            // Exisitiert das Zielverzeichnis
            // Wenn nicht --> Erstellen
            if (!Directory.Exists(destDirName))
            {
                Directory.CreateDirectory(destDirName);
            }

            // Dateien im Quellverzeichnis ermitteln
            // und in Zielverzeichnis kopieren
            IEnumerable<FileInfo> files = dir.EnumerateFiles();
            foreach (FileInfo file in files)
            {
                string temppath = Path.Combine(destDirName,
                                    file.Name);
                file.CopyTo(temppath, false);
            }

            // Sollen die Unterverzeichnisse kopiert werden?
            // Wenn ja --> rekursiver Aufruf
            if (copySubDirs)
            {
                foreach (DirectoryInfo subdir in dirs)
                {
                    string temppath = Path.Combine(destDirName,
                                        subdir.Name);
                    DirectoryCopy(subdir.FullName, temppath,
                      copySubDirs);
                }
            }
        }
    }
}
