﻿using System;
using System.IO;
using System.Security.Permissions;

namespace FileSystemWatcherSample
{
    class Program
    {
        public static void Main()
        {
            Run();
        }

        [PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
        private static void Run()
        {
            string path = @"c:\Temp\";

            // FileSystemWatcher-Instanz erzeugen
            using (FileSystemWatcher watcher = new FileSystemWatcher())
            {
                watcher.Path = path;

                // Nur *.txt-Dateien überwachen
                watcher.Filter = "*.txt";

                // Unterverzeichnisse überwachen?
                watcher.IncludeSubdirectories = false;

                // Events-registieren
                watcher.Changed += OnChanged;
                watcher.Created += OnChanged;
                watcher.Deleted += OnChanged;
                watcher.Renamed += OnRenamed;

                // Überwachung starten
                watcher.EnableRaisingEvents = true;

                // Warten bis der User das Programm beendet
                Console.WriteLine("Drücken Sie 'q' um das Programm zu beenden.");
                while (Console.Read() != 'q') ;
            }
        }

        /// <summary>
        /// Feststellen was an der Datei geändert wurde
        /// Operationen: Changed, Created oder Deleted
        /// </summary>
        private static void OnChanged(object source, FileSystemEventArgs e)
        {
            Console.WriteLine($"{DateTime.Now.ToLongTimeString()}: Datei {e.FullPath}, Operation: {e.ChangeType}");
        }

        /// <summary>
        /// Feststellen, dass Datei umbenannt wurde
        /// </summary>
        private static void OnRenamed(object source, RenamedEventArgs e)
        {
            // Feststellen, ob eine Datei umbenannt wurde
            Console.WriteLine($"{DateTime.Now.ToLongTimeString()}: Datei {e.OldFullPath} wurde zu {e.FullPath} umbenannt");
        }
    }
}
