﻿using System;
using System.IO;

namespace GetFolderSize
{
    public static class FileSystemUtils
    {
        public static long GetFolderSize(DirectoryInfo folder, bool ignoreUnauthorizedAccessException)
        {
            // Variable zur Ermittlung der Gesamtgröße
            // des Ordner
            long folderSize = 0;

            try
            {
                // Größe der Dateien innerhalb des Ordners
                // aufsummieren
                foreach (var fileInfo in folder.EnumerateFiles())
                {
                    folderSize += fileInfo.Length;
                }
            }
            catch (UnauthorizedAccessException ex)
            {
                if (!ignoreUnauthorizedAccessException)
                {
                    // Die Ausnahme nur dann weiter werfen, wenn diese nicht ignoriert
                    // werden soll
                    throw ex;
                }
            }

            // Unterordner ermitteln und die Methode rekursiv für jeden Unterordner
            // aufrufen und dabei die Variable folderSize hochzählen
            try
            {
                foreach (var subFolder in folder.EnumerateDirectories())
                {
                    folderSize += GetFolderSize(subFolder,
                    ignoreUnauthorizedAccessException);
                }
            }
            catch (UnauthorizedAccessException ex)
            {
                if (!ignoreUnauthorizedAccessException)
                {
                    // Die Ausnahme nur dann weiter werfen, wenn diese nicht ignoriert
                    // werden soll
                    throw ex;
                }
            }

            // Die ermittelte Ordnergröße zurückgeben
            return folderSize;
        }

        public static long GetFolderSize(string folderPath, bool ignoreUnauthorizedAccessException)
        {
            // Die rekursive Methode zur Ermittlung der Größe eines Ordners aufrufen
            return FileSystemUtils.GetFolderSize(new DirectoryInfo(folderPath), ignoreUnauthorizedAccessException);
        }
    }
}
